# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from nebius.api.nebius.common.v1 import operation_pb2 as nebius_dot_common_dot_v1_dot_operation__pb2
from nebius.api.nebius.msp.spark.v1alpha1 import session_pb2 as nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__pb2
from nebius.api.nebius.msp.spark.v1alpha1 import session_service_pb2 as nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2


class SessionServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Get = channel.unary_unary(
                '/nebius.msp.spark.v1alpha1.SessionService/Get',
                request_serializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.GetSessionRequest.SerializeToString,
                response_deserializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__pb2.Session.FromString,
                )
        self.GetByName = channel.unary_unary(
                '/nebius.msp.spark.v1alpha1.SessionService/GetByName',
                request_serializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.GetSessionByNameRequest.SerializeToString,
                response_deserializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__pb2.Session.FromString,
                )
        self.List = channel.unary_unary(
                '/nebius.msp.spark.v1alpha1.SessionService/List',
                request_serializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.ListSessionsRequest.SerializeToString,
                response_deserializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.ListSessionsResponse.FromString,
                )
        self.Create = channel.unary_unary(
                '/nebius.msp.spark.v1alpha1.SessionService/Create',
                request_serializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.CreateSessionRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )
        self.Delete = channel.unary_unary(
                '/nebius.msp.spark.v1alpha1.SessionService/Delete',
                request_serializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.DeleteSessionRequest.SerializeToString,
                response_deserializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
                )


class SessionServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Get(self, request, context):
        """Returns the specified session.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetByName(self, request, context):
        """Returns the specified session by name.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def List(self, request, context):
        """Retrieves a list of sessions.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Create(self, request, context):
        """Creates a session.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Delete(self, request, context):
        """Deletes a session.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SessionServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Get': grpc.unary_unary_rpc_method_handler(
                    servicer.Get,
                    request_deserializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.GetSessionRequest.FromString,
                    response_serializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__pb2.Session.SerializeToString,
            ),
            'GetByName': grpc.unary_unary_rpc_method_handler(
                    servicer.GetByName,
                    request_deserializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.GetSessionByNameRequest.FromString,
                    response_serializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__pb2.Session.SerializeToString,
            ),
            'List': grpc.unary_unary_rpc_method_handler(
                    servicer.List,
                    request_deserializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.ListSessionsRequest.FromString,
                    response_serializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.ListSessionsResponse.SerializeToString,
            ),
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.CreateSessionRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
            'Delete': grpc.unary_unary_rpc_method_handler(
                    servicer.Delete,
                    request_deserializer=nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.DeleteSessionRequest.FromString,
                    response_serializer=nebius_dot_common_dot_v1_dot_operation__pb2.Operation.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'nebius.msp.spark.v1alpha1.SessionService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class SessionService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.msp.spark.v1alpha1.SessionService/Get',
            nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.GetSessionRequest.SerializeToString,
            nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__pb2.Session.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetByName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.msp.spark.v1alpha1.SessionService/GetByName',
            nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.GetSessionByNameRequest.SerializeToString,
            nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__pb2.Session.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def List(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.msp.spark.v1alpha1.SessionService/List',
            nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.ListSessionsRequest.SerializeToString,
            nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.ListSessionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.msp.spark.v1alpha1.SessionService/Create',
            nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.CreateSessionRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Delete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nebius.msp.spark.v1alpha1.SessionService/Delete',
            nebius_dot_msp_dot_spark_dot_v1alpha1_dot_session__service__pb2.DeleteSessionRequest.SerializeToString,
            nebius_dot_common_dot_v1_dot_operation__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from collections.abc import Awaitable, Coroutine
from datetime import datetime, timedelta, timezone
from logging import getLogger
from typing import Any

from grpc.aio import AioRpcError
from grpc.aio import Channel as GRPCChannel
from grpc.aio._metadata import Metadata
from grpc_status import rpc_status

from nebius.aio.token.deferred_channel import DeferredChannel
from nebius.api.nebius.iam.v1.token_exchange_service_pb2_grpc import (
    TokenExchangeServiceStub,
)
from nebius.api.nebius.iam.v1.token_service_pb2 import CreateTokenResponse
from nebius.base.error import SDKError
from nebius.base.metadata import Authorization, Internal
from nebius.base.metadata import Metadata as NebiusMetadata
from nebius.base.sanitization import ellipsis_in_middle
from nebius.base.service_account.service_account import TokenRequester

from .options import OPTION_MAX_RETRIES
from .token import Bearer as ParentBearer
from .token import Receiver as ParentReceiver
from .token import Token

log = getLogger(__name__)


class UnsupportedResponseError(SDKError):
    def __init__(self, expected: str, resp: Any) -> None:
        super().__init__(
            f"Unsupported response received: expected {expected},"
            f" received {type(resp)}"
        )


class UnsupportedTokenTypeError(SDKError):
    def __init__(self, token_type: str) -> None:
        super().__init__(
            "Unsupported token received: expected Bearer," f" received {token_type}"
        )


class Receiver(ParentReceiver):
    def __init__(
        self,
        requester: TokenRequester,
        service: TokenExchangeServiceStub | Awaitable[TokenExchangeServiceStub],
        max_retries: int = 2,
    ) -> None:
        super().__init__()
        self._requester = requester
        self._svc = service
        self._max_retries = max_retries

        self._trial = 0

    def _raise_request_error(self, err: AioRpcError) -> None:
        initial_metadata = NebiusMetadata(err.initial_metadata())
        request_id = initial_metadata.get_one("x-request-id", "")
        trace_id = initial_metadata.get_one("x-trace-id", "")
        status = rpc_status.from_call(err)  # type: ignore
        from nebius.aio.service_error import RequestError, RequestStatusExtended

        if status is None:
            self._status = RequestStatusExtended(
                code=err.code(),
                message=err.details(),
                details=[],
                service_errors=[],
                request_id=request_id,
                trace_id=trace_id,
            )
            raise RequestError(self._status) from None

        self._status = RequestStatusExtended.from_rpc_status(  # type: ignore[unused-ignore]
            status,
            trace_id=trace_id,
            request_id=request_id,
        )
        raise RequestError(self._status) from None

    async def _fetch(
        self, timeout: float | None = None, options: dict[str, str] | None = None
    ) -> Token:
        self._trial += 1
        req = self._requester.get_exchange_token_request()

        now = datetime.now(timezone.utc)

        md = Metadata()
        md.add(Internal.AUTHORIZATION.lower(), Authorization.DISABLE)

        log.debug(f"fetching new token, attempt: {self._trial}, timeout: {timeout}")

        ret = None
        try:
            if isinstance(self._svc, Awaitable):
                self._svc = await self._svc
            ret = await self._svc.Exchange(req, metadata=md, timeout=timeout)  # type: ignore[unused-ignore]
        except AioRpcError as e:
            self._raise_request_error(e)
        if not isinstance(ret, CreateTokenResponse):
            raise UnsupportedResponseError(CreateTokenResponse.__name__, ret)

        if ret.token_type != "Bearer":  # noqa: S105 — not a password
            raise UnsupportedTokenTypeError(ret.token_type)

        log.debug(
            f"token fetched: {ellipsis_in_middle(ret.access_token)},"
            f" expires in: {ret.expires_in} seconds."
        )
        return Token(
            token=ret.access_token, expiration=now + timedelta(seconds=ret.expires_in)
        )

    def can_retry(
        self,
        err: Exception,
        options: dict[str, str] | None = None,
    ) -> bool:
        max_retries = self._max_retries
        if options is not None and OPTION_MAX_RETRIES in options:
            value = options[OPTION_MAX_RETRIES]
            try:
                max_retries = int(value)
            except ValueError as err:
                log.error(f"option {OPTION_MAX_RETRIES} is not valid integer: {err=}")
        if self._trial >= max_retries:
            log.debug("token max retries reached, cannot retry")
            return False
        return True


class Bearer(ParentBearer):
    def __init__(
        self,
        requester: TokenRequester,
        channel: GRPCChannel | DeferredChannel | None = None,
        max_retries: int = 2,
    ) -> None:
        super().__init__()
        self._requester = requester
        self._max_retries = max_retries

        self._svc: (
            TokenExchangeServiceStub
            | Coroutine[Any, Any, TokenExchangeServiceStub]
            | None
        ) = None
        self.set_channel(channel)

    def set_channel(self, channel: GRPCChannel | DeferredChannel | None) -> None:
        """
        Set the gRPC channel for the bearer.
        """
        if isinstance(channel, Awaitable):  # type: ignore[unused-ignore]

            async def token_exchange_service_stub() -> TokenExchangeServiceStub:
                chan = await channel
                if not isinstance(chan, GRPCChannel):  # type: ignore[unused-ignore]
                    raise TypeError(f"Expected GRPCChannel, got {type(chan)} instead.")
                return TokenExchangeServiceStub(chan)  # type: ignore

            self._svc = token_exchange_service_stub()

        elif channel is not None:
            self._svc = TokenExchangeServiceStub(channel)  # type:ignore
        else:
            self._svc = None

    def receiver(self) -> Receiver:
        if self._svc is None:
            raise ValueError("gRPC channel is not set for the bearer.")
        return Receiver(self._requester, self._svc, max_retries=self._max_retries)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius.common.v1 import operation_pb2 as _operation_pb2
from nebius.api.nebius.storage.v1alpha1 import transfer_pb2 as _transfer_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GetTransferRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class ListTransfersRequest(_message.Message):
    __slots__ = ["parent_id", "page_size", "page_token"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    page_size: int
    page_token: str
    def __init__(self, parent_id: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ...) -> None: ...

class ListTransfersResponse(_message.Message):
    __slots__ = ["items", "next_page_token"]
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[_transfer_pb2.Transfer]
    next_page_token: str
    def __init__(self, items: _Optional[_Iterable[_Union[_transfer_pb2.Transfer, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...

class CreateTransferRequest(_message.Message):
    __slots__ = ["metadata", "spec"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: _transfer_pb2.TransferSpec
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[_transfer_pb2.TransferSpec, _Mapping]] = ...) -> None: ...

class UpdateTransferRequest(_message.Message):
    __slots__ = ["metadata", "spec"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: _transfer_pb2.TransferSpec
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[_transfer_pb2.TransferSpec, _Mapping]] = ...) -> None: ...

class StopTransferRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class ResumeTransferRequest(_message.Message):
    __slots__ = ["id"]
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class DeleteTransferRequest(_message.Message):
    __slots__ = ["id", "resource_version"]
    ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_VERSION_FIELD_NUMBER: _ClassVar[int]
    id: str
    resource_version: int
    def __init__(self, id: _Optional[str] = ..., resource_version: _Optional[int] = ...) -> None: ...

class GetIterationHistoryRequest(_message.Message):
    __slots__ = ["transfer_id", "page_size", "page_token"]
    TRANSFER_ID_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    transfer_id: str
    page_size: int
    page_token: str
    def __init__(self, transfer_id: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ...) -> None: ...

class GetIterationHistoryResponse(_message.Message):
    __slots__ = ["iterations", "next_page_token"]
    ITERATIONS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    iterations: _containers.RepeatedCompositeFieldContainer[_transfer_pb2.TransferIteration]
    next_page_token: str
    def __init__(self, iterations: _Optional[_Iterable[_Union[_transfer_pb2.TransferIteration, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...



from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Request(_message.Message):
    __slots__ = ["request_id", "parameters", "idempotency_id", "trace_id"]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    PARAMETERS_FIELD_NUMBER: _ClassVar[int]
    IDEMPOTENCY_ID_FIELD_NUMBER: _ClassVar[int]
    TRACE_ID_FIELD_NUMBER: _ClassVar[int]
    request_id: str
    parameters: _struct_pb2.Struct
    idempotency_id: str
    trace_id: str
    def __init__(self, request_id: _Optional[str] = ..., parameters: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., idempotency_id: _Optional[str] = ..., trace_id: _Optional[str] = ...) -> None: ...
